from __future__ import print_function
from tigershark.X12 import message


def pprint(x12_message, level=0, indent=" "):
    """ Pretty print the parsed message.

    :param loop: The `X12.message.X12Message` to print.
    :type loop: `X12.message.X12Message`
    :param level: The current indentation level.
    :type level: int
    :param indent: The character(s) to use for indentation.
    :type indent: String
    """
    if x12_message.parts is not None:
        for segment in x12_message.parts:
            if isinstance(segment, message.X12Segment):
                print("{indent}{name} - {elements}".format(
                    indent=level * indent,
                    name=segment.name,
                    elements=segment.elements))
            else:
                print("{indent}#### {name} ####".format(
                    indent=(level + 1) * indent,
                    name=segment.name))
                pprint(segment, level + 1, indent)



from decimal import Decimal

from tigershark.facade import ElementAccess
from tigershark.facade import Facade
from tigershark.facade import Money
from tigershark.facade import X12LoopBridge
from tigershark.facade import enum
from tigershark.facade.enums.common import id_code_qualifier
from tigershark.facade.enums.common import reference_id_qualifier
from tigershark.facade.enums.remittance import claim_adjustment_reasons


class ClaimAdjustment(X12LoopBridge):
    group = ElementAccess("CAS", 1, x12type=enum(
        {"CO": "Contractual Obligation",
         "CR": "Correction and/or reversal",
         "OA": "Other Adjustment",
         "PI": "Payor initiated adjustment",
         "PR": "Patient Responsibility"}))

    reason_1 = ElementAccess("CAS", 2,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_1 = ElementAccess("CAS", 3, x12type=Money)
    quantity_1 = ElementAccess("CAS", 4)

    reason_2 = ElementAccess("CAS", 5,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_2 = ElementAccess("CAS", 6, x12type=Money)
    quantity_2 = ElementAccess("CAS", 7)

    reason_3 = ElementAccess("CAS", 8,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_3 = ElementAccess("CAS", 9, x12type=Money)
    quantity_3 = ElementAccess("CAS", 10)

    reason_4 = ElementAccess("CAS", 11,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_4 = ElementAccess("CAS", 12, x12type=Money)
    quantity_4 = ElementAccess("CAS", 13)

    reason_5 = ElementAccess("CAS", 14,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_5 = ElementAccess("CAS", 15, x12type=Money)
    quantity_5 = ElementAccess("CAS", 16)

    reason_6 = ElementAccess("CAS", 17,
            x12type=enum(claim_adjustment_reasons, raw_unknowns=True))
    amount_6 = ElementAccess("CAS", 18, x12type=Money)
    quantity_6 = ElementAccess("CAS", 19)

    def total_amount(self, reason=None):
        s = Decimal('0.0')
        for i in range(1, 7):
            val = self.__getattribute__("amount_%s" % i)
            reason_tuple = self.__getattribute__("reason_%s" % i)
            if reason is None or (reason_tuple is not None and
                    reason_tuple[0] is not None and
                    reason_tuple[0] == reason):
                s += val
        return s

    def total_quantity(self, reason=None):
        s = 0.0
        for i in range(1, 7):
            a = self.__getattribute__("quantity_%s" % i)
            if a is not None:
                try:
                    s += float(a)
                except:
                    pass
        return s

    def all_reasons_iter(self):
        for i in range(1, 7):
            reason_tuple = self.__getattribute__("reason_%s" % i)
            if reason_tuple is not None:
                yield reason_tuple

    def __init__(self, aLoop, qualifier, *args, **kwargs):
        self.qualifier = qualifier
        super(ClaimAdjustment, self).__init__(aLoop, *args, **kwargs)


class ContactDetails(X12LoopBridge):
    name = ElementAccess("N1", 2)
    id_qualifier = ElementAccess("N1", 3, x12type=enum({
            "XV": "Health Care Financing Administration National Plan ID",
            "FI": "Federal Taxpayer Identification Number",
            "XX": "Health Care Financing Administration National Provider ID"
            }))
    id = ElementAccess("N1", 4)
    addr1 = ElementAccess("N3", 1)
    addr2 = ElementAccess("N3", 2)
    city = ElementAccess("N4", 1)
    state = ElementAccess("N4", 2)
    zip = ElementAccess("N4", 3)
    country_code = ElementAccess("N4", 4)
    location_type = ElementAccess("N4", 5, x12type=enum({
        "CY": "County/Parish",
        "FI": "Federal Information Processing Standards (FIPS) 55 (Named "
                "Populated Places)"}, raw_unknowns=True))
    location_id = ElementAccess("N4", 6)
    contact_code = ElementAccess("PER", 1, x12type=enum({
        "IC": "Information Contact"}, raw_unknowns=True))
    contact_name = ElementAccess("PER", 2)
    contact_edi = ElementAccess("PER", oneOf=("ED", (3, 4), (5, 6), (7, 8)))
    contact_email = ElementAccess("PER", oneOf=("EM", (3, 4), (5, 6), (7, 8)))
    contact_fax = ElementAccess("PER", oneOf=("FX", (3, 4), (5, 6), (7, 8)))
    contact_home_phone = ElementAccess("PER",
            oneOf=("HP", (3, 4), (5, 6), (7, 8)))
    contact_work_phone = ElementAccess("PER",
            oneOf=("WP", (3, 4), (5, 6), (7, 8)))
    contact_phone = ElementAccess("PER", oneOf=("TE", (3, 4), (5, 6), (7, 8)))
    contact_phone_ext = ElementAccess("PER",
            oneOf=("EX", (3, 4), (5, 6), (7, 8)))


class NamedEntity(X12LoopBridge):
    entity_identifier = ElementAccess("NM1", 1, x12type=enum({
            "03": "Dependent",
            "1P": "Provider",
            "2B": "Third-Party Administrator",
            "36": "Employer",
            "80": "Hospital",
            "FA": "Facility",
            "GP": "Gateway Provider",
            "IL": "Insured",
            "P5": "Plan Sponsor",
            "PR": "Payer",
            "QC": "Patient"}))
    entity_type = ElementAccess("NM1", 2, x12type=enum({
            "1": "Person",
            "2": "Non-Person Entity"}))

    last_name = ElementAccess("NM1", 3)
    org_name = ElementAccess("NM1", 3)
    first_name = ElementAccess("NM1", 4)
    middle_initial = ElementAccess("NM1", 5)
    suffix = ElementAccess("NM1", 7)

    id_code = ElementAccess("NM1", 9)
    id_code_qual = ElementAccess("NM1", 8, x12type=enum(id_code_qualifier))

    def is_person(self):
        return self.entity_type[0] == "1"

    def is_organization(self):
        return self.entity_type[0] == "2"

    def __init__(self, aLoop, qualifier=None, *args, **kwargs):
        if qualifier:
            self.qualifier = qualifier
        super(NamedEntity, self).__init__(aLoop)
        #NOTE: Sometimes this can be multiple Elements...
        self.contact_details = ContactDetails(aLoop, *args, **kwargs)


class ReferenceID(Facade, X12LoopBridge):
    reference_id_qualifier = ElementAccess("REF", 1,
            x12type=enum(reference_id_qualifier))
    reference_id = ElementAccess("REF", 2)
    description = ElementAccess("REF", 3)



from tigershark import X12VersionTuple
from tigershark.facade import D8
from tigershark.facade import ElementAccess
from tigershark.facade import Facade
from tigershark.facade import TM
from tigershark.facade import X12LoopBridge


class ControlHeaders(Facade):

    def __init__(self, x12_message):
        super(ControlHeaders, self).__init__()
        isa_loops = x12_message.descendant('LOOP', name='ISA_LOOP')
        self.interchange_controls = [InterchangeControlHeader(loop) for loop in isa_loops]


class InterchangeControlHeader(X12LoopBridge):

    authorization_information_qualifier = ElementAccess("ISA", 1)
    authorization_information = ElementAccess("ISA", 2)

    security_information_qualifier = ElementAccess("ISA", 3)
    security_information = ElementAccess("ISA", 4)

    interchange_sender_id_qualifier = ElementAccess("ISA", 5)
    interchange_sender_id = ElementAccess("ISA", 6)

    interchange_receiver_id_qualifier = ElementAccess("ISA", 7)
    interchange_receiver_id = ElementAccess("ISA", 8)

    interchange_date = ElementAccess("ISA", 9, x12type=D8)
    interchange_time = ElementAccess("ISA", 10, x12type=TM)

    interchange_control_standards_id = ElementAccess("ISA", 11)
    interchange_control_version_number = ElementAccess("ISA", 12)
    interchange_control_number = ElementAccess("ISA", 13)

    acknowledgement_requested = ElementAccess("ISA", 14)
    test_indicator = ElementAccess("ISA", 15)
    subelement_separator = ElementAccess("ISA", 16)

    def __init__(self, x12_message):
        super(InterchangeControlHeader, self).__init__(x12_message)
        gs_loops = x12_message.descendant('LOOP', name='GS_LOOP')
        self.functional_groups = [FunctionalGroupHeader(loop) for loop in gs_loops]


class FunctionalGroupHeader(X12LoopBridge):

    functional_id_code = ElementAccess("GS", 1)

    application_sender_code = ElementAccess("GS", 2)
    application_receiver_code = ElementAccess("GS", 3)

    date = ElementAccess("GS", 4, x12type=D8)
    time = ElementAccess("GS", 5, x12type=TM)

    group_control_number = ElementAccess("GS", 6)

    responsible_agency_code = ElementAccess("GS", 7)

    version_indicator_code = ElementAccess("GS", 8)

    def __init__(self, x12_message):
        super(FunctionalGroupHeader, self).__init__(x12_message)
        st_loops = x12_message.descendant('LOOP', name='ST_LOOP')
        self.transaction_sets = [
            TransactionSetHeader(self, st_loop)
            for st_loop in st_loops]

    @property
    def version_tuple(self):
        """
        Return a tuple of (version, release, subrelease) numbers.

        Returns None if no version is given or has a different format.
        """
        if self.responsible_agency_code == 'X':
            return X12VersionTuple(
                version=int(self.version_indicator_code[0:3]),
                release=int(self.version_indicator_code[3:5]),
                subrelease=int(self.version_indicator_code[5:6]),
                industry_identifier_code=self.version_indicator_code[6:],
            )


class TransactionSetHeader(X12LoopBridge):

    transaction_set_identifier_code = ElementAccess("ST", 1)

    transaction_set_control_number = ElementAccess("ST", 2)

    def __init__(self, functional_group, x12_message):
        super(TransactionSetHeader, self).__init__(x12_message)
        self.functional_group = functional_group



#
# Generated by TigerShark.tools.convertPyX12 on 2012-07-10 16:30:03.879137
#
from tigershark.X12.parse import Message, Loop, Segment, Composite, Element, Properties
parsed_841_1100 = Loop( u'1100', Properties(looptype='',repeat=u'>1',pos=u'040',req_sit=u'R',desc=u'Code List Source'),
Segment( u'N1', Properties(syntax=u'R0203 P0304',req_sit=u'S',repeat=u'1',pos=u'120',desc=u'Code List Source'),
  Element( u'N101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'0F'] ) ),
  Element( u'N102', Properties(desc=u'Name', req_sit=u'R', data_type=(u'AN',u'1',u'60'), position=2,
    codes=[] ) ),
  Element( u'N103', Properties(desc=u'Identification Code Qualifier', req_sit=u'N', data_type=(u'ID',u'1',u'2'), position=3,
    codes=[] ) ),
  Element( u'N104', Properties(desc=u'Identification Code', req_sit=u'N', data_type=(u'AN',u'2',u'80'), position=4,
    codes=[] ) ),
  Element( u'N105', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[] ) ),
  Element( u'N106', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=6,
    codes=[] ) ),
),
)
parsed_841_1000 = Loop( u'1000', Properties(looptype='',repeat=u'>1',pos=u'010',req_sit=u'R',desc=u'Code List'),
Segment( u'SPI', Properties(syntax=u'P0203',req_sit=u'R',repeat=u'1',pos=u'020',desc=u'Code List'),
  Element( u'SPI01', Properties(desc=u'Security Level Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'00'] ) ),
  Element( u'SPI02', Properties(desc=u'Reference Identification Qualifier', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=2,
    codes=[] ) ),
  Element( u'SPI03', Properties(desc=u'Reference Identification', req_sit=u'N', data_type=(u'AN',u'1',u'50'), position=3,
    codes=[] ) ),
  Element( u'SPI04', Properties(desc=u'Entity Title', req_sit=u'R', data_type=(None,None,None), position=4,
    codes=[] ) ),
  Element( u'SPI05', Properties(desc=u'Entity Purpose', req_sit=u'R', data_type=(None,None,None), position=5,
    codes=[] ) ),
  Element( u'SPI06', Properties(desc=u'Entity Status Code', req_sit=u'N', data_type=(None,None,None), position=6,
    codes=[] ) ),
  Element( u'SPI07', Properties(desc=u'Transaction Set Purpose Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[] ) ),
  Element( u'SPI08', Properties(desc=u'Report Type Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=8,
    codes=[] ) ),
  Element( u'SPI09', Properties(desc=u'Security Level Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=9,
    codes=[] ) ),
  Element( u'SPI10', Properties(desc=u'Agency Qualifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'SPI11', Properties(desc=u'Source Subqualifier', req_sit=u'N', data_type=(u'AN',u'1',u'15'), position=11,
    codes=[] ) ),
  Element( u'SPI12', Properties(desc=u'Assigned Number', req_sit=u'N', data_type=(u'N0',u'1',u'6'), position=12,
    codes=[] ) ),
  Element( u'SPI13', Properties(desc=u'Certification Type Code', req_sit=u'N', data_type=(u'ID',u'1',u'1'), position=13,
    codes=[] ) ),
  Element( u'SPI14', Properties(desc=u'Proposal Data Detail Identifier Code', req_sit=u'N', data_type=(None,None,None), position=14,
    codes=[] ) ),
  Element( u'SPI15', Properties(desc=u'Hierarchical Structure Code', req_sit=u'N', data_type=(u'ID',u'4',u'4'), position=15,
    codes=[] ) ),
),
Segment( u'RDT', Properties(syntax=u'C0102 L030405 C0605',req_sit=u'R',repeat=u'>1',pos=u'030',desc=u'Release Date'),
  Element( u'RDT01', Properties(desc=u'Revision Level Code', req_sit=u'N', data_type=(None,None,None), position=1,
    codes=[] ) ),
  Element( u'RDT02', Properties(desc=u'Revision Value', req_sit=u'N', data_type=(None,None,None), position=2,
    codes=[] ) ),
  Element( u'RDT03', Properties(desc=u'Date/Time Qualifier', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=3,
    codes=[u'171'] ) ),
  Element( u'RDT04', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=4,
    codes=[] ) ),
  Element( u'RDT05', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=5,
    codes=[] ) ),
  Element( u'RDT06', Properties(desc=u'Time Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=6,
    codes=[] ) ),
),
Segment( u'RDT', Properties(syntax=u'C0102 L030405 C0605',req_sit=u'S',repeat=u'1',pos=u'030',desc=u'Effective Date'),
  Element( u'RDT01', Properties(desc=u'Revision Level Code', req_sit=u'N', data_type=(None,None,None), position=1,
    codes=[] ) ),
  Element( u'RDT02', Properties(desc=u'Revision Value', req_sit=u'N', data_type=(None,None,None), position=2,
    codes=[] ) ),
  Element( u'RDT03', Properties(desc=u'Date/Time Qualifier', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=3,
    codes=[u'007'] ) ),
  Element( u'RDT04', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=4,
    codes=[] ) ),
  Element( u'RDT05', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=5,
    codes=[] ) ),
  Element( u'RDT06', Properties(desc=u'Time Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=6,
    codes=[] ) ),
),
parsed_841_1100,
)
parsed_841_HEADER = Loop( u'HEADER', Properties(looptype=u'wrapper',repeat=u'1',pos=u'010',req_sit=u'R',desc=u'Table 1 - Header'),
parsed_841_1000,
)
parsed_841_2100 = Loop( u'2100', Properties(looptype='',repeat=u'>1',pos=u'010',req_sit=u'R',desc=u'Code List Value and Definition'),
Segment( u'SPI', Properties(syntax=u'P0203',req_sit=u'R',repeat=u'1',pos=u'020',desc=u'Code List Value and Definition'),
  Element( u'SPI01', Properties(desc=u'Security Level Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'00'] ) ),
  Element( u'SPI02', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=2,
    codes=[u'ZZ'] ) ),
  Element( u'SPI03', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=3,
    codes=[] ) ),
  Element( u'SPI04', Properties(desc=u'Entity Title', req_sit=u'R', data_type=(None,None,None), position=4,
    codes=[] ) ),
  Element( u'SPI05', Properties(desc=u'Entity Purpose', req_sit=u'N', data_type=(None,None,None), position=5,
    codes=[] ) ),
  Element( u'SPI06', Properties(desc=u'Entity Status Code', req_sit=u'N', data_type=(None,None,None), position=6,
    codes=[] ) ),
  Element( u'SPI07', Properties(desc=u'Transaction Set Purpose Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[] ) ),
  Element( u'SPI08', Properties(desc=u'Report Type Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=8,
    codes=[] ) ),
  Element( u'SPI09', Properties(desc=u'Security Level Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=9,
    codes=[] ) ),
  Element( u'SPI10', Properties(desc=u'Agency Qualifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'SPI11', Properties(desc=u'Source Subqualifier', req_sit=u'N', data_type=(u'AN',u'1',u'15'), position=11,
    codes=[] ) ),
  Element( u'SPI12', Properties(desc=u'Assigned Number', req_sit=u'N', data_type=(u'N0',u'1',u'6'), position=12,
    codes=[] ) ),
  Element( u'SPI13', Properties(desc=u'Certification Type Code', req_sit=u'N', data_type=(u'ID',u'1',u'1'), position=13,
    codes=[] ) ),
  Element( u'SPI14', Properties(desc=u'Proposal Data Detail Identifier Code', req_sit=u'N', data_type=(None,None,None), position=14,
    codes=[] ) ),
  Element( u'SPI15', Properties(desc=u'Hierarchical Structure Code', req_sit=u'N', data_type=(u'ID',u'4',u'4'), position=15,
    codes=[] ) ),
),
Segment( u'MSG', Properties(syntax=u'C0302',req_sit=u'S',repeat=u'>1',pos=u'050',desc=u'Additional Text'),
  Element( u'MSG01', Properties(desc=u'Free-form Message Text', req_sit=u'R', data_type=(u'AN',u'1',u'264'), position=1,
    codes=[] ) ),
  Element( u'MSG02', Properties(desc=u'Printer Carriage Control Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=2,
    codes=[] ) ),
  Element( u'MSG03', Properties(desc=u'Number', req_sit=u'N', data_type=(u'N0',u'1',u'9'), position=3,
    codes=[] ) ),
),
)
parsed_841_2000 = Loop( u'2000', Properties(looptype='',repeat=u'>1',pos=u'010',req_sit=u'R',desc=u'Code List Header'),
Segment( u'HL', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Code List Header'),
  Element( u'HL01', Properties(desc=u'Hierarchical ID Number', req_sit=u'R', data_type=(u'AN',u'1',u'12'), position=1,
    codes=[u'1'] ) ),
  Element( u'HL02', Properties(desc=u'Hierarchical Parent ID Number', req_sit=u'N', data_type=(u'AN',u'1',u'12'), position=2,
    codes=[] ) ),
  Element( u'HL03', Properties(desc=u'Hierarchical Level Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=3,
    codes=[u'I'] ) ),
  Element( u'HL04', Properties(desc=u'Hierarchical Child Code', req_sit=u'N', data_type=(u'ID',u'1',u'1'), position=4,
    codes=[] ) ),
),
parsed_841_2100,
)
parsed_841_DETAIL = Loop( u'DETAIL', Properties(looptype=u'wrapper',repeat=u'>1',pos=u'020',req_sit=u'S',desc=u'Table 2 - Detail'),
parsed_841_2000,
)
parsed_841_ST_LOOP = Loop( u'ST_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'020',req_sit=u'R',desc=u'Transaction Set Header'),
Segment( u'ST', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'005',desc=u'Transaction Set Header'),
  Element( u'ST01', Properties(desc=u'Transaction Set Identifier Code', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=1,
    codes=[u'841'] ) ),
  Element( u'ST02', Properties(desc=u'Transaction Set Control Number', req_sit=u'R', data_type=(u'AN',u'4',u'9'), position=2,
    codes=[] ) ),
),
parsed_841_HEADER,
parsed_841_DETAIL,
Segment( u'SE', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'210',desc=u'Transaction Set Trailer'),
  Element( u'SE01', Properties(desc=u'Number of Included Segments', req_sit=u'R', data_type=(u'N0',u'1',u'10'), position=1,
    codes=[] ) ),
  Element( u'SE02', Properties(desc=u'Transaction Set Control Number', req_sit=u'R', data_type=(u'AN',u'4',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_841_GS_LOOP = Loop( u'GS_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'020',req_sit=u'R',desc=u'Functional Group Header'),
Segment( u'GS', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Functional Group Header'),
  Element( u'GS01', Properties(desc=u'Functional Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'SP'] ) ),
  Element( u'GS02', Properties(desc=u'Application Senders Code', req_sit=u'R', data_type=(u'AN',u'2',u'15'), position=2,
    codes=[] ) ),
  Element( u'GS03', Properties(desc=u'124', req_sit=u'R', data_type=(u'AN',u'2',u'15'), position=3,
    codes=[] ) ),
  Element( u'GS04', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=4,
    codes=[] ) ),
  Element( u'GS05', Properties(desc=u'Time', req_sit=u'R', data_type=(u'TM',u'4',u'8'), position=5,
    codes=[] ) ),
  Element( u'GS06', Properties(desc=u'Group Control Number', req_sit=u'R', data_type=(u'N0',u'1',u'9'), position=6,
    codes=[] ) ),
  Element( u'GS07', Properties(desc=u'Responsible Agency Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=7,
    codes=[u'X'] ) ),
  Element( u'GS08', Properties(desc=u'Version / Release / Industry Identifier Code', req_sit=u'R', data_type=(u'AN',u'1',u'12'), position=8,
    codes=[u'004010XXXC'] ) ),
),
parsed_841_ST_LOOP,
Segment( u'GE', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'030',desc=u'Functional Group Trailer'),
  Element( u'GE01', Properties(desc=u'97', req_sit=u'R', data_type=(u'N0',u'1',u'6'), position=1,
    codes=[] ) ),
  Element( u'GE02', Properties(desc=u'Group Control Number', req_sit=u'R', data_type=(u'N0',u'1',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_841_ISA_LOOP = Loop( u'ISA_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'001',req_sit=u'R',desc=u'Interchange Control Header'),
Segment( u'ISA', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Interchange Control Header'),
  Element( u'ISA01', Properties(desc=u'I01', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'00', u'03'] ) ),
  Element( u'ISA02', Properties(desc=u'I02', req_sit=u'R', data_type=(u'AN',u'10',u'10'), position=2,
    codes=[] ) ),
  Element( u'ISA03', Properties(desc=u'I03', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=3,
    codes=[u'00', u'01'] ) ),
  Element( u'ISA04', Properties(desc=u'I04', req_sit=u'R', data_type=(u'AN',u'10',u'10'), position=4,
    codes=[] ) ),
  Element( u'ISA05', Properties(desc=u'I05', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[u'01', u'14', u'20', u'27', u'28', u'29', u'30', u'33', u'ZZ'] ) ),
  Element( u'ISA06', Properties(desc=u'I06', req_sit=u'R', data_type=(u'AN',u'15',u'15'), position=6,
    codes=[] ) ),
  Element( u'ISA07', Properties(desc=u'I05', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[u'01', u'14', u'20', u'27', u'28', u'29', u'30', u'33', u'ZZ'] ) ),
  Element( u'ISA08', Properties(desc=u'I07', req_sit=u'R', data_type=(u'AN',u'15',u'15'), position=8,
    codes=[] ) ),
  Element( u'ISA09', Properties(desc=u'I08', req_sit=u'R', data_type=(u'DT',u'6',u'6'), position=9,
    codes=[] ) ),
  Element( u'ISA10', Properties(desc=u'I09', req_sit=u'R', data_type=(u'TM',u'4',u'4'), position=10,
    codes=[] ) ),
  Element( u'ISA11', Properties(desc=u'I10', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=11,
    codes=[u'U'] ) ),
  Element( u'ISA12', Properties(desc=u'I11', req_sit=u'R', data_type=(u'ID',u'5',u'5'), position=12,
    codes=[u'00401'] ) ),
  Element( u'ISA13', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=13,
    codes=[] ) ),
  Element( u'ISA14', Properties(desc=u'I13', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=14,
    codes=[u'0', u'1'] ) ),
  Element( u'ISA15', Properties(desc=u'I14', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=15,
    codes=[u'P', u'T'] ) ),
  Element( u'ISA16', Properties(desc=u'I15', req_sit=u'R', data_type=(u'AN',u'1',u'1'), position=16,
    codes=[] ) ),
),
parsed_841_GS_LOOP,
Segment( u'TA1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'020',desc=u'Interchange Acknowledgement'),
  Element( u'TA101', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=1,
    codes=[] ) ),
  Element( u'TA102', Properties(desc=u'I08', req_sit=u'R', data_type=(u'DT',u'6',u'6'), position=2,
    codes=[] ) ),
  Element( u'TA103', Properties(desc=u'I09', req_sit=u'R', data_type=(u'TM',u'4',u'4'), position=3,
    codes=[] ) ),
  Element( u'TA104', Properties(desc=u'I17', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=4,
    codes=[u'A', u'E', u'R'] ) ),
  Element( u'TA105', Properties(desc=u'I18', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=5,
    codes=[u'000', u'001', u'002', u'003', u'004', u'005', u'006', u'007', u'008', u'009', u'010', u'011', u'012', u'013', u'014', u'015', u'016', u'017', u'018', u'019', u'020', u'021', u'022', u'023', u'024', u'025', u'026', u'027', u'028', u'029', u'030', u'031'] ) ),
),
Segment( u'IEA', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'030',desc=u'Interchange Control Trailer'),
  Element( u'IEA01', Properties(desc=u'I16', req_sit=u'R', data_type=(u'N0',u'1',u'5'), position=1,
    codes=[] ) ),
  Element( u'IEA02', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_841 = Message( u'841', Properties(desc=u'HIPAA Related Code Lists'),
parsed_841_ISA_LOOP,
)




#!/usr/bin/env python
"""The ``X12.map.dj`` package emits Django class definitions for Segment definitions.

See :ref:`traversal` for notes on the **Visitor** design pattern.

..  autoclass:: DjangoModelVisitor
    :members:

..  autoclass:: DjangoAdminVisitor
    :members:
"""
from tigershark.X12.parse import StructureVisitor
from tigershark.X12.parse import StopDescent

class DjangoModelVisitor( StructureVisitor ):
    """Emit Django class definitions for Segments.

    Extension to :class:`X12.parse.StructureVisitor`
    """
    def __init__( self, *args, **kw ):
        super( DjangoModelVisitor, self ).__init__( *args, **kw )
        self.result= ['from web.claims.models import X12Segment','from django.db import models','']
        self.segTypes= set()
    def preSegment( self, aSegment, indent=0 ):
        """Examine this Segment: is it a new type?"""
        if aSegment.name in self.segTypes:
            raise StopDescent( "Already seen this Segment Type" )
        self.segTypes.add( aSegment.name )
        self.result.append( 'class Segment_%s(models.Model):' % ( aSegment.name, ) )
        self.result.append( '    """%r"""' % ( aSegment.props,) )
        self.result.append( '    segment = models.ForeignKey( X12Segment )' )
    def postSegment( self, aSegment, indent=0 ):
        self.result.append( "    def unmarshall( self, segmentToken, cs=':' ):" )
        children= aSegment.structure
        for pos in range(len(children)):
            if len(children[pos].structure):
                # Composites have further decomposition
                self.result.append( "        # composite= segmentToken[%d].split(cs)" % ( pos+1, ) )
                subChildren= children[pos].structure
                for c in range(len(subChildren)):
                    self.result.append( "        self.%s = segmentToken.subElt(%d,cs,%d)" % (
                        self._fixName(subChildren[c].name), pos+1, c ))
            else:
                # Elements are atomic
                nm= children[pos].name
                self.result.append( "        self.%s = segmentToken.elt(%d)" % ( nm, pos+1 ) )
        self.result.append( "" )
    def preComposite( self, aComposite, indent=0 ):
        """Report this Composite."""
        self.result.append( '    # <Composite %r, %r, ' % (
            aComposite.name, aComposite.props ) )
    def _fixName( self, name ):
        return name.replace("-","_")
    def postElement( self, anElement, indent=0 ):
        """Report this Element."""
        elt= anElement
        opts= { 'max_length' : (16 if elt.max_len is None else int(elt.max_len)) }
        if elt.req_sit != "R":
            opts['blank']= True
            opts['null']= True
        # XXX - Use elt.type_name
        code= '    %s = models.CharField( %s ) # %r %r'
        optStr= ", ".join( [ '%s=%s' % (n,v) for n,v in opts.items() ] )
        self.result.append( code % ( self._fixName(elt.name), optStr, elt.props.data_type, elt.props.desc, ) )
    def getSource( self, appname='claims_837' ):
        return "\n".join( self.result )

class DjangoAdminVisitor( StructureVisitor ):
    """Emit Django admin definitions for Segments.

    Extension to :class:`X12.parse.StructureVisitor`
    """
    def __init__( self, *args, **kw ):
        super( DjangoAdminVisitor, self ).__init__( *args, **kw )
        self.result= ['from django.contrib import admin','']
        self.segTypes= set()
    def preSegment( self, aSegment, indent ):
        if aSegment.name in self.segTypes:
            raise StopDescent( "Already seen this Segment Type" )
        self.segTypes.add( aSegment.name )
    def getSource( self, appname='claims_837' ):
        def emitter( segTypes ):
            for aSegment in self.segTypes:
                yield 'from web.{0}.models import {1}'.format(appname,aSegment)
                yield 'admin.site.register( {1} )'.format(appname,aSegment)
        return "\n".join( emitter(self.segTypes) )



#
# Generated by TigerShark.tools.convertPyX12 on 2012-10-15 15:31:48.524770
#
from tigershark.X12.parse import Message, Loop, Segment, Composite, Element, Properties
parsed_835_1000A = Loop( u'1000A', Properties(looptype='',repeat=u'1',pos=u'080',req_sit=u'R',desc=u'Payer Identification'),
Segment( u'N1', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'080',desc=u'Payer Identification'),
  Element( u'N101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'PR'] ) ),
  Element( u'N102', Properties(desc=u'Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=2,
    codes=[] ) ),
  Element( u'N103', Properties(desc=u'Identification Code Qualifier', req_sit=u'S', data_type=(u'ID',u'1',u'2'), position=3,
    codes=[u'XV'] ) ),
  Element( u'N104', Properties(desc=u'Identification Code', req_sit=u'S', data_type=(u'AN',u'2',u'80'), position=4,
    codes=[] ) ),
  Element( u'N105', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[] ) ),
  Element( u'N106', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=6,
    codes=[] ) ),
),
Segment( u'N3', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'100',desc=u'Payer Address'),
  Element( u'N301', Properties(desc=u'Address Information', req_sit=u'R', data_type=(u'AN',u'1',u'55'), position=1,
    codes=[] ) ),
  Element( u'N302', Properties(desc=u'Address Information', req_sit=u'S', data_type=(u'AN',u'1',u'55'), position=2,
    codes=[] ) ),
),
Segment( u'N4', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'110',desc=u'Payer City, State, ZIP Code'),
  Element( u'N401', Properties(desc=u'City Name', req_sit=u'R', data_type=(u'AN',u'2',u'30'), position=1,
    codes=[] ) ),
  Element( u'N402', Properties(desc=u'State or Province Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=2,
    codes=[] ) ),
  Element( u'N403', Properties(desc=u'Postal Code', req_sit=u'R', data_type=(u'ID',u'3',u'15'), position=3,
    codes=[] ) ),
  Element( u'N404', Properties(desc=u'Country Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=4,
    codes=[] ) ),
  Element( u'N405', Properties(desc=u'Location Qualifier', req_sit=u'N', data_type=(u'ID',u'1',u'2'), position=5,
    codes=[] ) ),
  Element( u'N406', Properties(desc=u'Location Identifier', req_sit=u'N', data_type=(u'AN',u'1',u'30'), position=6,
    codes=[] ) ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'4',pos=u'120',desc=u'Additional Payer Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'2U', u'EO', u'HI', u'NF'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'PER', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'130',desc=u'Payer Contact Information'),
  Element( u'PER01', Properties(desc=u'Contact Function Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'CX'] ) ),
  Element( u'PER02', Properties(desc=u'Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=2,
    codes=[] ) ),
  Element( u'PER03', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=3,
    codes=[u'EM', u'FX', u'TE'] ) ),
  Element( u'PER04', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=4,
    codes=[] ) ),
  Element( u'PER05', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[u'EM', u'EX', u'FX', u'TE'] ) ),
  Element( u'PER06', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=6,
    codes=[] ) ),
  Element( u'PER07', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[u'EX'] ) ),
  Element( u'PER08', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=8,
    codes=[] ) ),
  Element( u'PER09', Properties(desc=u'Contact Inquiry Reference', req_sit=u'N', data_type=(u'AN',u'1',u'20'), position=9,
    codes=[] ) ),
),
)
parsed_835_1000B = Loop( u'1000B', Properties(looptype='',repeat=u'1',pos=u'080',req_sit=u'R',desc=u'Payee Identification'),
Segment( u'N1', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'080',desc=u'Payee Identification'),
  Element( u'N101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'PE'] ) ),
  Element( u'N102', Properties(desc=u'Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=2,
    codes=[] ) ),
  Element( u'N103', Properties(desc=u'Identification Code Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=3,
    codes=[u'FI', u'XX'] ) ),
  Element( u'N104', Properties(desc=u'Identification Code', req_sit=u'R', data_type=(u'AN',u'2',u'80'), position=4,
    codes=[] ) ),
  Element( u'N105', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[] ) ),
  Element( u'N106', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=6,
    codes=[] ) ),
),
Segment( u'N3', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'100',desc=u'Payee Address'),
  Element( u'N301', Properties(desc=u'Address Information', req_sit=u'R', data_type=(u'AN',u'1',u'55'), position=1,
    codes=[] ) ),
  Element( u'N302', Properties(desc=u'Address Information', req_sit=u'S', data_type=(u'AN',u'1',u'55'), position=2,
    codes=[] ) ),
),
Segment( u'N4', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'110',desc=u'Payee City, State, ZIP Code'),
  Element( u'N401', Properties(desc=u'City Name', req_sit=u'R', data_type=(u'AN',u'2',u'30'), position=1,
    codes=[] ) ),
  Element( u'N402', Properties(desc=u'State or Province Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=2,
    codes=[] ) ),
  Element( u'N403', Properties(desc=u'Postal Code', req_sit=u'R', data_type=(u'ID',u'3',u'15'), position=3,
    codes=[] ) ),
  Element( u'N404', Properties(desc=u'Country Code', req_sit=u'S', data_type=(u'ID',u'2',u'3'), position=4,
    codes=[] ) ),
  Element( u'N405', Properties(desc=u'Location Qualifier', req_sit=u'N', data_type=(u'ID',u'1',u'2'), position=5,
    codes=[] ) ),
  Element( u'N406', Properties(desc=u'Location Identifier', req_sit=u'N', data_type=(u'AN',u'1',u'30'), position=6,
    codes=[] ) ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'>1',pos=u'120',desc=u'Payee Additional Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'0B', u'1A', u'1B', u'1C', u'1D', u'1E', u'1F', u'1G', u'1H', u'D3', u'G2', u'N5', u'PQ', u'TJ'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
)
parsed_835_HEADER = Loop( u'HEADER', Properties(looptype=u'wrapper',repeat=u'1',pos=u'015',req_sit=u'R',desc=u'Table 1 - Header'),
Segment( u'BPR', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'020',desc=u'Financial Information'),
  Element( u'BPR01', Properties(desc=u'Transaction Handling Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=1,
    codes=[u'C', u'D', u'H', u'I', u'P', u'U', u'X'] ) ),
  Element( u'BPR02', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'BPR03', Properties(desc=u'Credit/Debit Flag Code', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=3,
    codes=[u'C', u'D'] ) ),
  Element( u'BPR04', Properties(desc=u'Payment Method Code', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=4,
    codes=[u'ACH', u'BOP', u'CHK', u'FWT', u'NON'] ) ),
  Element( u'BPR05', Properties(desc=u'Payment Format Code', req_sit=u'S', data_type=(u'ID',u'1',u'10'), position=5,
    codes=[u'CCP', u'CTX'] ) ),
  Element( u'BPR06', Properties(desc=u'(DFI) ID Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=6,
    codes=[u'01', u'04'] ) ),
  Element( u'BPR07', Properties(desc=u'(DFI) Identification Number', req_sit=u'S', data_type=(u'AN',u'3',u'12'), position=7,
    codes=[] ) ),
  Element( u'BPR08', Properties(desc=u'Account Number Qualifier', req_sit=u'S', data_type=(u'ID',u'1',u'3'), position=8,
    codes=[u'DA'] ) ),
  Element( u'BPR09', Properties(desc=u'Account Number', req_sit=u'S', data_type=(u'AN',u'1',u'35'), position=9,
    codes=[] ) ),
  Element( u'BPR10', Properties(desc=u'Originating Company Identifier', req_sit=u'S', data_type=(u'AN',u'10',u'10'), position=10,
    codes=[] ) ),
  Element( u'BPR11', Properties(desc=u'Originating Company Supplemental Code', req_sit=u'S', data_type=(u'AN',u'9',u'9'), position=11,
    codes=[] ) ),
  Element( u'BPR12', Properties(desc=u'(DFI) ID Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=12,
    codes=[u'01', u'04'] ) ),
  Element( u'BPR13', Properties(desc=u'(DFI) Identification Number', req_sit=u'S', data_type=(u'AN',u'3',u'12'), position=13,
    codes=[] ) ),
  Element( u'BPR14', Properties(desc=u'Account Number Qualifier', req_sit=u'S', data_type=(u'ID',u'1',u'3'), position=14,
    codes=[u'DA', u'SG'] ) ),
  Element( u'BPR15', Properties(desc=u'Account Number', req_sit=u'S', data_type=(u'AN',u'1',u'35'), position=15,
    codes=[] ) ),
  Element( u'BPR16', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=16,
    codes=[] ) ),
  Element( u'BPR17', Properties(desc=u'Business Function Code', req_sit=u'N', data_type=(u'ID',u'1',u'3'), position=17,
    codes=[] ) ),
  Element( u'BPR18', Properties(desc=u'(DFI) ID Number Qualifier', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=18,
    codes=[] ) ),
  Element( u'BPR19', Properties(desc=u'(DFI) Identification Number', req_sit=u'N', data_type=(u'AN',u'3',u'12'), position=19,
    codes=[] ) ),
  Element( u'BPR20', Properties(desc=u'Account Number Qualifier', req_sit=u'N', data_type=(u'ID',u'1',u'3'), position=20,
    codes=[] ) ),
  Element( u'BPR21', Properties(desc=u'Account Number', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=21,
    codes=[] ) ),
),
Segment( u'TRN', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'040',desc=u'Reassociation Trace Number'),
  Element( u'TRN01', Properties(desc=u'Trace Type Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=1,
    codes=[u'1'] ) ),
  Element( u'TRN02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'TRN03', Properties(desc=u'Originating Company Identifier', req_sit=u'R', data_type=(u'AN',u'10',u'10'), position=3,
    codes=[] ) ),
  Element( u'TRN04', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=4,
    codes=[] ) ),
),
Segment( u'CUR', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'050',desc=u'Foreign Currency Information'),
  Element( u'CUR01', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'PR'] ) ),
  Element( u'CUR02', Properties(desc=u'Currency Code', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=2,
    codes=[] ) ),
  Element( u'CUR03', Properties(desc=u'Exchange Rate', req_sit=u'S', data_type=(u'R',u'4',u'10'), position=3,
    codes=[] ) ),
  Element( u'CUR04', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=4,
    codes=[] ) ),
  Element( u'CUR05', Properties(desc=u'Currency Code', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=5,
    codes=[] ) ),
  Element( u'CUR06', Properties(desc=u'Currency Market/Exchange Code', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=6,
    codes=[] ) ),
  Element( u'CUR07', Properties(desc=u'Date/Time Qualifier', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=7,
    codes=[] ) ),
  Element( u'CUR08', Properties(desc=u'Date', req_sit=u'N', data_type=(u'DT',u'8',u'8'), position=8,
    codes=[] ) ),
  Element( u'CUR09', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=9,
    codes=[] ) ),
  Element( u'CUR10', Properties(desc=u'Date/Time Qualifier', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=10,
    codes=[] ) ),
  Element( u'CUR11', Properties(desc=u'Date', req_sit=u'N', data_type=(u'DT',u'8',u'8'), position=11,
    codes=[] ) ),
  Element( u'CUR12', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=12,
    codes=[] ) ),
  Element( u'CUR13', Properties(desc=u'Date/Time Qualifier', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=13,
    codes=[] ) ),
  Element( u'CUR14', Properties(desc=u'Date', req_sit=u'N', data_type=(u'DT',u'8',u'8'), position=14,
    codes=[] ) ),
  Element( u'CUR15', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=15,
    codes=[] ) ),
  Element( u'CUR16', Properties(desc=u'Date/Time Qualifier', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=16,
    codes=[] ) ),
  Element( u'CUR17', Properties(desc=u'Date', req_sit=u'N', data_type=(u'DT',u'8',u'8'), position=17,
    codes=[] ) ),
  Element( u'CUR18', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=18,
    codes=[] ) ),
  Element( u'CUR19', Properties(desc=u'Date/Time Qualifier', req_sit=u'N', data_type=(u'ID',u'3',u'3'), position=19,
    codes=[] ) ),
  Element( u'CUR20', Properties(desc=u'Date', req_sit=u'N', data_type=(u'DT',u'8',u'8'), position=20,
    codes=[] ) ),
  Element( u'CUR21', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=21,
    codes=[] ) ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'060',desc=u'Receiver Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'EV'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'060',desc=u'Version Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'F2'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'DTM', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'070',desc=u'Production Date'),
  Element( u'DTM01', Properties(desc=u'Date/Time Qualifier', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=1,
    codes=[u'405'] ) ),
  Element( u'DTM02', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=2,
    codes=[] ) ),
  Element( u'DTM03', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=3,
    codes=[] ) ),
  Element( u'DTM04', Properties(desc=u'Time Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=4,
    codes=[] ) ),
  Element( u'DTM05', Properties(desc=u'Date Time Period Format Qualifier', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=5,
    codes=[] ) ),
  Element( u'DTM06', Properties(desc=u'Date Time Period', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=6,
    codes=[] ) ),
),
parsed_835_1000A,
parsed_835_1000B,
)
parsed_835_2110 = Loop( u'2110', Properties(looptype='',repeat=u'999',pos=u'070',req_sit=u'S',desc=u'Service Payment Information'),
Segment( u'SVC', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'070',desc=u'Service Payment Information'),
  Composite( u'C003', Properties(req_sit=u'R',refdes='',seq=u'01',desc=u'Composite Medical Procedure Identifier'),
    Element( u'SVC01-01', Properties(desc=u'Product/Service ID Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[u'AD', u'ER', u'HC', u'ID', u'IV', u'N4', u'NU', u'RB', u'ZZ'] ) ),
    Element( u'SVC01-02', Properties(desc=u'Product/Service ID', req_sit=u'R', data_type=(u'AN',u'1',u'48'), position=1,
      codes=[] ) ),
    Element( u'SVC01-03', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=2,
      codes=[] ) ),
    Element( u'SVC01-04', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=3,
      codes=[] ) ),
    Element( u'SVC01-05', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=4,
      codes=[] ) ),
    Element( u'SVC01-06', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=5,
      codes=[] ) ),
    Element( u'SVC01-07', Properties(desc=u'Description', req_sit=u'S', data_type=(u'AN',u'1',u'80'), position=6,
      codes=[] ) ),
  ),
  Element( u'SVC02', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'SVC03', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=3,
    codes=[] ) ),
  Element( u'SVC04', Properties(desc=u'Product/Service ID', req_sit=u'S', data_type=(u'AN',u'1',u'48'), position=4,
    codes=[] ) ),
  Element( u'SVC05', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=5,
    codes=[] ) ),
  Composite( u'C003', Properties(req_sit=u'S',refdes='',seq=u'06',desc=u'Composite Medical Procedure Identifier'),
    Element( u'SVC06-01', Properties(desc=u'Product/Service ID Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[u'AD', u'ER', u'HC', u'ID', u'IV', u'N4', u'NU', u'RB', u'ZZ'] ) ),
    Element( u'SVC06-02', Properties(desc=u'Product/Service ID', req_sit=u'R', data_type=(u'AN',u'1',u'48'), position=1,
      codes=[] ) ),
    Element( u'SVC06-03', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=2,
      codes=[] ) ),
    Element( u'SVC06-04', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=3,
      codes=[] ) ),
    Element( u'SVC06-05', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=4,
      codes=[] ) ),
    Element( u'SVC06-06', Properties(desc=u'Procedure Modifier', req_sit=u'S', data_type=(u'AN',u'2',u'2'), position=5,
      codes=[] ) ),
    Element( u'SVC06-07', Properties(desc=u'Description', req_sit=u'S', data_type=(u'AN',u'1',u'80'), position=6,
      codes=[] ) ),
  ),
  Element( u'SVC07', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=7,
    codes=[] ) ),
),
Segment( u'DTM', Properties(syntax='',req_sit=u'S',repeat=u'3',pos=u'080',desc=u'Service Date'),
  Element( u'DTM01', Properties(desc=u'Date/Time Qualifier', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=1,
    codes=[u'150', u'151', u'472'] ) ),
  Element( u'DTM02', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=2,
    codes=[] ) ),
  Element( u'DTM03', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=3,
    codes=[] ) ),
  Element( u'DTM04', Properties(desc=u'Time Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=4,
    codes=[] ) ),
  Element( u'DTM05', Properties(desc=u'Date Time Period Format Qualifier', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=5,
    codes=[] ) ),
  Element( u'DTM06', Properties(desc=u'Date Time Period', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=6,
    codes=[] ) ),
),
Segment( u'CAS', Properties(syntax='',req_sit=u'S',repeat=u'99',pos=u'090',desc=u'Service Adjustment'),
  Element( u'CAS01', Properties(desc=u'Claim Adjustment Group Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=1,
    codes=[u'CO', u'CR', u'OA', u'PI', u'PR'] ) ),
  Element( u'CAS02', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'1',u'5'), position=2,
    codes=[] ) ),
  Element( u'CAS03', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=3,
    codes=[] ) ),
  Element( u'CAS04', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=4,
    codes=[] ) ),
  Element( u'CAS05', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=5,
    codes=[] ) ),
  Element( u'CAS06', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Element( u'CAS07', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=7,
    codes=[] ) ),
  Element( u'CAS08', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=8,
    codes=[] ) ),
  Element( u'CAS09', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
  Element( u'CAS10', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=10,
    codes=[] ) ),
  Element( u'CAS11', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=11,
    codes=[] ) ),
  Element( u'CAS12', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=12,
    codes=[] ) ),
  Element( u'CAS13', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=13,
    codes=[] ) ),
  Element( u'CAS14', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=14,
    codes=[] ) ),
  Element( u'CAS15', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=15,
    codes=[] ) ),
  Element( u'CAS16', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=16,
    codes=[] ) ),
  Element( u'CAS17', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=17,
    codes=[] ) ),
  Element( u'CAS18', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=18,
    codes=[] ) ),
  Element( u'CAS19', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=19,
    codes=[] ) ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'7',pos=u'100',desc=u'Service Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'1S', u'6R', u'BB', u'E9', u'G1', u'G3', u'LU', u'RB'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'10',pos=u'100',desc=u'Rendering Provider Information'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'1A', u'1B', u'1C', u'1D', u'1G', u'1H', u'1J', u'HPI', u'SY', u'TJ'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'AMT', Properties(syntax='',req_sit=u'S',repeat=u'12',pos=u'110',desc=u'Service Supplemental Amount'),
  Element( u'AMT01', Properties(desc=u'Amount Qualifier Code', req_sit=u'R', data_type=(u'ID',u'1',u'3'), position=1,
    codes=[u'B6', u'DY', u'KH', u'NE', u'T', u'T2', u'ZK', u'ZL', u'ZM', u'ZN', u'ZO'] ) ),
  Element( u'AMT02', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'AMT03', Properties(desc=u'Credit/Debit Flag Code', req_sit=u'N', data_type=(u'ID',u'1',u'1'), position=3,
    codes=[] ) ),
),
Segment( u'QTY', Properties(syntax='',req_sit=u'S',repeat=u'6',pos=u'120',desc=u'Service Supplemental Quantity'),
  Element( u'QTY01', Properties(desc=u'Quantity Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'NE', u'ZK', u'ZL', u'ZM', u'ZN', u'ZO'] ) ),
  Element( u'QTY02', Properties(desc=u'Quantity', req_sit=u'R', data_type=(u'R',u'1',u'15'), position=2,
    codes=[] ) ),
  Composite( u'C001', Properties(req_sit=u'N',refdes='',seq=u'03',desc=u'Composite Unit of Measure'),
  ),
  Element( u'QTY04', Properties(desc=u'Free-form Information', req_sit=u'N', data_type=(u'AN',u'1',u'30'), position=4,
    codes=[] ) ),
),
Segment( u'LQ', Properties(syntax='',req_sit=u'S',repeat=u'99',pos=u'130',desc=u'Health Care Remark Codes'),
  Element( u'LQ01', Properties(desc=u'Code List Qualifier Code', req_sit=u'R', data_type=(u'ID',u'1',u'3'), position=1,
    codes=[u'HE', u'RX'] ) ),
  Element( u'LQ02', Properties(desc=u'Industry Code', req_sit=u'R', data_type=(u'AN',u'1',u'30'), position=2,
    codes=[] ) ),
),
)
parsed_835_2100 = Loop( u'2100', Properties(looptype='',repeat=u'>1',pos=u'010',req_sit=u'R',desc=u'Claim Payment Information'),
Segment( u'CLP', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Claim Payment Information'),
  Element( u'CLP01', Properties(desc=u"Claim Submitter's Identifier", req_sit=u'R', data_type=(u'AN',u'1',u'38'), position=1,
    codes=[] ) ),
  Element( u'CLP02', Properties(desc=u'Claim Status Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=2,
    codes=[] ) ),
  Element( u'CLP03', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=3,
    codes=[] ) ),
  Element( u'CLP04', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=4,
    codes=[] ) ),
  Element( u'CLP05', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=5,
    codes=[] ) ),
  Element( u'CLP06', Properties(desc=u'Claim Filing Indicator Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=6,
    codes=[u'12', u'13', u'14', u'15', u'16', u'AM', u'CH', u'DS', u'HM', u'LM', u'MA', u'MB', u'MC', u'OF', u'TV', u'VA', u'WC', u'ZZ'] ) ),
  Element( u'CLP07', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=7,
    codes=[] ) ),
  Element( u'CLP08', Properties(desc=u'Facility Code Value', req_sit=u'S', data_type=(u'AN',u'1',u'2'), position=8,
    codes=[] ) ),
  Element( u'CLP09', Properties(desc=u'Claim Frequency Type Code', req_sit=u'S', data_type=(u'ID',u'1',u'1'), position=9,
    codes=[u'0', u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'A', u'B', u'C', u'D', u'E', u'F', u'G', u'H', u'I', u'J', u'K', u'L', u'M', u'N', u'O', u'X', u'Y', u'Z'] ) ),
  Element( u'CLP10', Properties(desc=u'Patient Status Code', req_sit=u'N', data_type=(u'ID',u'1',u'2'), position=10,
    codes=[] ) ),
  Element( u'CLP11', Properties(desc=u'Diagnosis Related Group (DRG) Code', req_sit=u'S', data_type=(u'ID',u'1',u'4'), position=11,
    codes=[] ) ),
  Element( u'CLP12', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=12,
    codes=[] ) ),
  Element( u'CLP13', Properties(desc=u'Percentage as Decimal', req_sit=u'S', data_type=(u'R',u'1',u'10'), position=13,
    codes=[] ) ),
),
Segment( u'CAS', Properties(syntax='',req_sit=u'S',repeat=u'99',pos=u'020',desc=u'Claim Adjustment'),
  Element( u'CAS01', Properties(desc=u'Claim Adjustment Group Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=1,
    codes=[u'CO', u'CR', u'OA', u'PI', u'PR'] ) ),
  Element( u'CAS02', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'1',u'5'), position=2,
    codes=[] ) ),
  Element( u'CAS03', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=3,
    codes=[] ) ),
  Element( u'CAS04', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=4,
    codes=[] ) ),
  Element( u'CAS05', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=5,
    codes=[] ) ),
  Element( u'CAS06', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Element( u'CAS07', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=7,
    codes=[] ) ),
  Element( u'CAS08', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=8,
    codes=[] ) ),
  Element( u'CAS09', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
  Element( u'CAS10', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=10,
    codes=[] ) ),
  Element( u'CAS11', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=11,
    codes=[] ) ),
  Element( u'CAS12', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=12,
    codes=[] ) ),
  Element( u'CAS13', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=13,
    codes=[] ) ),
  Element( u'CAS14', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=14,
    codes=[] ) ),
  Element( u'CAS15', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=15,
    codes=[] ) ),
  Element( u'CAS16', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=16,
    codes=[] ) ),
  Element( u'CAS17', Properties(desc=u'Claim Adjustment Reason Code', req_sit=u'S', data_type=(u'ID',u'1',u'5'), position=17,
    codes=[] ) ),
  Element( u'CAS18', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=18,
    codes=[] ) ),
  Element( u'CAS19', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=19,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'030',desc=u'Patient Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'QC'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'1'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'R', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'R', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'S', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'S', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'S', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'34', u'HN', u'II', u'MI', u'MR'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'S', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'030',desc=u'Insured Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'IL'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'1', u'2'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'S', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'S', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'S', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'34', u'HN', u'MI'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'R', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'030',desc=u'Corrected Patient/Insured Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'74'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'1', u'2'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'S', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'S', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'S', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'S', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'C'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'S', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'030',desc=u'Service Provider Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'82'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'1', u'2'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'S', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'S', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'S', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'BD', u'BS', u'FI', u'MC', u'PC', u'SL', u'UP', u'XX'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'R', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'030',desc=u'Crossover Carrier Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'TT'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'2'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'R', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'N', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'AD', u'FI', u'NI', u'PI', u'PP', u'XV'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'R', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'NM1', Properties(syntax='',req_sit=u'S',repeat=u'2',pos=u'030',desc=u'Corrected Priority Payer Name'),
  Element( u'NM101', Properties(desc=u'Entity Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'PR'] ) ),
  Element( u'NM102', Properties(desc=u'Entity Type Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=2,
    codes=[u'2'] ) ),
  Element( u'NM103', Properties(desc=u'Name Last or Organization Name', req_sit=u'R', data_type=(u'AN',u'1',u'60'), position=3,
    codes=[] ) ),
  Element( u'NM104', Properties(desc=u'Name First', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=4,
    codes=[] ) ),
  Element( u'NM105', Properties(desc=u'Name Middle', req_sit=u'N', data_type=(u'AN',u'1',u'25'), position=5,
    codes=[] ) ),
  Element( u'NM106', Properties(desc=u'Name Prefix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=6,
    codes=[] ) ),
  Element( u'NM107', Properties(desc=u'Name Suffix', req_sit=u'N', data_type=(u'AN',u'1',u'10'), position=7,
    codes=[] ) ),
  Element( u'NM108', Properties(desc=u'Identification Code Qualifier', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=8,
    codes=[u'AD', u'FI', u'NI', u'PI', u'PP', u'XV'] ) ),
  Element( u'NM109', Properties(desc=u'Identification Code', req_sit=u'R', data_type=(u'AN',u'2',u'80'), position=9,
    codes=[] ) ),
  Element( u'NM110', Properties(desc=u'Entity Relationship Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=10,
    codes=[] ) ),
  Element( u'NM111', Properties(desc=u'Entity Identifier Code', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=11,
    codes=[] ) ),
),
Segment( u'MIA', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'033',desc=u'Inpatient Adjudication Information'),
  Element( u'MIA01', Properties(desc=u'Quantity', req_sit=u'R', data_type=(u'R',u'1',u'15'), position=1,
    codes=[] ) ),
  Element( u'MIA02', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=2,
    codes=[] ) ),
  Element( u'MIA03', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=3,
    codes=[] ) ),
  Element( u'MIA04', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=4,
    codes=[] ) ),
  Element( u'MIA05', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=5,
    codes=[] ) ),
  Element( u'MIA06', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Element( u'MIA07', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=7,
    codes=[] ) ),
  Element( u'MIA08', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=8,
    codes=[] ) ),
  Element( u'MIA09', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
  Element( u'MIA10', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=10,
    codes=[] ) ),
  Element( u'MIA11', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=11,
    codes=[] ) ),
  Element( u'MIA12', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=12,
    codes=[] ) ),
  Element( u'MIA13', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=13,
    codes=[] ) ),
  Element( u'MIA14', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=14,
    codes=[] ) ),
  Element( u'MIA15', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=15,
    codes=[] ) ),
  Element( u'MIA16', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=16,
    codes=[] ) ),
  Element( u'MIA17', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=17,
    codes=[] ) ),
  Element( u'MIA18', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=18,
    codes=[] ) ),
  Element( u'MIA19', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=19,
    codes=[] ) ),
  Element( u'MIA20', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=20,
    codes=[] ) ),
  Element( u'MIA21', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=21,
    codes=[] ) ),
  Element( u'MIA22', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=22,
    codes=[] ) ),
  Element( u'MIA23', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=23,
    codes=[] ) ),
  Element( u'MIA24', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=24,
    codes=[] ) ),
),
Segment( u'MOA', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'035',desc=u'Outpatient Adjudication Information'),
  Element( u'MOA01', Properties(desc=u'Percentage as Decimal', req_sit=u'S', data_type=(u'R',u'1',u'10'), position=1,
    codes=[] ) ),
  Element( u'MOA02', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'MOA03', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=3,
    codes=[] ) ),
  Element( u'MOA04', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=4,
    codes=[] ) ),
  Element( u'MOA05', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=5,
    codes=[] ) ),
  Element( u'MOA06', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=6,
    codes=[] ) ),
  Element( u'MOA07', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=7,
    codes=[] ) ),
  Element( u'MOA08', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=8,
    codes=[] ) ),
  Element( u'MOA09', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'5',pos=u'040',desc=u'Other Claim Related Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'1L', u'1W', u'9A', u'9C', u'A6', u'BB', u'CE', u'EA', u'F8', u'G1', u'G3', u'IG', u'SY'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'REF', Properties(syntax='',req_sit=u'S',repeat=u'10',pos=u'040',desc=u'Rendering Provider Identification'),
  Element( u'REF01', Properties(desc=u'Reference Identification Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'3'), position=1,
    codes=[u'1A', u'1B', u'1C', u'1D', u'1G', u'1H', u'D3', u'G2'] ) ),
  Element( u'REF02', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=2,
    codes=[] ) ),
  Element( u'REF03', Properties(desc=u'Description', req_sit=u'N', data_type=(u'AN',u'1',u'80'), position=3,
    codes=[] ) ),
  Composite( u'C040', Properties(req_sit=u'N',refdes='',seq=u'04',desc=u'Reference Identifier'),
  ),
),
Segment( u'DTM', Properties(syntax='',req_sit=u'S',repeat=u'4',pos=u'050',desc=u'Claim Date'),
  Element( u'DTM01', Properties(desc=u'Date/Time Qualifier', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=1,
    codes=[u'036', u'050', u'232', u'233'] ) ),
  Element( u'DTM02', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=2,
    codes=[] ) ),
  Element( u'DTM03', Properties(desc=u'Time', req_sit=u'N', data_type=(u'TM',u'4',u'8'), position=3,
    codes=[] ) ),
  Element( u'DTM04', Properties(desc=u'Time Code', req_sit=u'N', data_type=(u'ID',u'2',u'2'), position=4,
    codes=[] ) ),
  Element( u'DTM05', Properties(desc=u'Date Time Period Format Qualifier', req_sit=u'N', data_type=(u'ID',u'2',u'3'), position=5,
    codes=[] ) ),
  Element( u'DTM06', Properties(desc=u'Date Time Period', req_sit=u'N', data_type=(u'AN',u'1',u'35'), position=6,
    codes=[] ) ),
),
Segment( u'PER', Properties(syntax='',req_sit=u'S',repeat=u'3',pos=u'060',desc=u'Claim Contact Information'),
  Element( u'PER01', Properties(desc=u'Contact Function Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'CX'] ) ),
  Element( u'PER02', Properties(desc=u'Name', req_sit=u'S', data_type=(u'AN',u'1',u'60'), position=2,
    codes=[] ) ),
  Element( u'PER03', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=3,
    codes=[u'EM', u'FX', u'TE'] ) ),
  Element( u'PER04', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=4,
    codes=[] ) ),
  Element( u'PER05', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[u'EM', u'EX', u'FX', u'TE'] ) ),
  Element( u'PER06', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=6,
    codes=[] ) ),
  Element( u'PER07', Properties(desc=u'Communication Number Qualifier', req_sit=u'S', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[u'EX'] ) ),
  Element( u'PER08', Properties(desc=u'Communication Number', req_sit=u'S', data_type=(u'AN',u'1',u'256'), position=8,
    codes=[] ) ),
  Element( u'PER09', Properties(desc=u'Contact Inquiry Reference', req_sit=u'N', data_type=(u'AN',u'1',u'20'), position=9,
    codes=[] ) ),
),
Segment( u'AMT', Properties(syntax='',req_sit=u'S',repeat=u'14',pos=u'062',desc=u'Claim Supplemental Information'),
  Element( u'AMT01', Properties(desc=u'Amount Qualifier Code', req_sit=u'R', data_type=(u'ID',u'1',u'3'), position=1,
    codes=[u'I', u'T', u'AU', u'D8', u'DY', u'F5', u'NL', u'T2', u'ZK', u'ZL', u'ZM', u'ZN', u'ZO', u'ZZ'] ) ),
  Element( u'AMT02', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'AMT03', Properties(desc=u'Credit/Debit Flag Code', req_sit=u'N', data_type=(u'ID',u'1',u'1'), position=3,
    codes=[] ) ),
),
Segment( u'QTY', Properties(syntax='',req_sit=u'S',repeat=u'15',pos=u'064',desc=u'Claim Supplemental Information Quantity'),
  Element( u'QTY01', Properties(desc=u'Quantity Qualifier', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'CA', u'CD', u'LA', u'LE', u'NA', u'NE', u'NR', u'OU', u'PS', u'VS', u'ZK', u'ZL', u'ZM', u'ZN', u'ZO'] ) ),
  Element( u'QTY02', Properties(desc=u'Quantity', req_sit=u'R', data_type=(u'R',u'1',u'15'), position=2,
    codes=[] ) ),
  Composite( u'C001', Properties(req_sit=u'N',refdes='',seq=u'03',desc=u'Composite Unit of Measure'),
  ),
  Element( u'QTY04', Properties(desc=u'Free-form Information', req_sit=u'N', data_type=(u'AN',u'1',u'30'), position=4,
    codes=[] ) ),
),
parsed_835_2110,
)
parsed_835_2000 = Loop( u'2000', Properties(looptype='',repeat=u'>1',pos=u'003',req_sit=u'S',desc=u'Header Number'),
Segment( u'LX', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'003',desc=u'Header Number'),
  Element( u'LX01', Properties(desc=u'Assigned Number', req_sit=u'R', data_type=(u'N0',u'1',u'6'), position=1,
    codes=[] ) ),
),
Segment( u'TS3', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'005',desc=u'Provider Summary Information'),
  Element( u'TS301', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=1,
    codes=[] ) ),
  Element( u'TS302', Properties(desc=u'Facility Code Value', req_sit=u'R', data_type=(u'AN',u'1',u'2'), position=2,
    codes=[] ) ),
  Element( u'TS303', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=3,
    codes=[] ) ),
  Element( u'TS304', Properties(desc=u'Quantity', req_sit=u'R', data_type=(u'R',u'1',u'15'), position=4,
    codes=[] ) ),
  Element( u'TS305', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=5,
    codes=[] ) ),
  Element( u'TS306', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Element( u'TS307', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=7,
    codes=[] ) ),
  Element( u'TS308', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=8,
    codes=[] ) ),
  Element( u'TS309', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
  Element( u'TS310', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=10,
    codes=[] ) ),
  Element( u'TS311', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=11,
    codes=[] ) ),
  Element( u'TS312', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=12,
    codes=[] ) ),
  Element( u'TS313', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=13,
    codes=[] ) ),
  Element( u'TS314', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=14,
    codes=[] ) ),
  Element( u'TS315', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=15,
    codes=[] ) ),
  Element( u'TS316', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=16,
    codes=[] ) ),
  Element( u'TS317', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=17,
    codes=[] ) ),
  Element( u'TS318', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=18,
    codes=[] ) ),
  Element( u'TS319', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=19,
    codes=[] ) ),
  Element( u'TS320', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=20,
    codes=[] ) ),
  Element( u'TS321', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=21,
    codes=[] ) ),
  Element( u'TS322', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=22,
    codes=[] ) ),
  Element( u'TS323', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=23,
    codes=[] ) ),
  Element( u'TS324', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=24,
    codes=[] ) ),
),
Segment( u'TS2', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'007',desc=u'Provider Supplemental Summary Information'),
  Element( u'TS201', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=1,
    codes=[] ) ),
  Element( u'TS202', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=2,
    codes=[] ) ),
  Element( u'TS203', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=3,
    codes=[] ) ),
  Element( u'TS204', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=4,
    codes=[] ) ),
  Element( u'TS205', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=5,
    codes=[] ) ),
  Element( u'TS206', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Element( u'TS207', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=7,
    codes=[] ) ),
  Element( u'TS208', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=8,
    codes=[] ) ),
  Element( u'TS209', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=9,
    codes=[] ) ),
  Element( u'TS210', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=10,
    codes=[] ) ),
  Element( u'TS211', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=11,
    codes=[] ) ),
  Element( u'TS212', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=12,
    codes=[] ) ),
  Element( u'TS213', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=13,
    codes=[] ) ),
  Element( u'TS214', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=14,
    codes=[] ) ),
  Element( u'TS215', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=15,
    codes=[] ) ),
  Element( u'TS216', Properties(desc=u'Quantity', req_sit=u'S', data_type=(u'R',u'1',u'15'), position=16,
    codes=[] ) ),
  Element( u'TS217', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=17,
    codes=[] ) ),
  Element( u'TS218', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=18,
    codes=[] ) ),
  Element( u'TS219', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=19,
    codes=[] ) ),
),
parsed_835_2100,
)
parsed_835_DETAIL = Loop( u'DETAIL', Properties(looptype=u'wrapper',repeat=u'>1',pos=u'020',req_sit=u'S',desc=u'Table 2 - Detail'),
parsed_835_2000,
)
parsed_835_FOOTER = Loop( u'FOOTER', Properties(looptype=u'wrapper',repeat=u'>1',pos=u'030',req_sit=u'S',desc=u'Table 3 - Footer'),
Segment( u'PLB', Properties(syntax='',req_sit=u'S',repeat=u'>1',pos=u'010',desc=u'Provider Adjustment'),
  Element( u'PLB01', Properties(desc=u'Reference Identification', req_sit=u'R', data_type=(u'AN',u'1',u'50'), position=1,
    codes=[] ) ),
  Element( u'PLB02', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=2,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'R',refdes='',seq=u'03',desc=u'Adjustment Identifier'),
    Element( u'PLB03-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB03-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB04', Properties(desc=u'Monetary Amount', req_sit=u'R', data_type=(u'R',u'1',u'18'), position=4,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'S',refdes='',seq=u'05',desc=u'Adjustment Identifier'),
    Element( u'PLB05-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB05-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB06', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=6,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'S',refdes='',seq=u'07',desc=u'Adjustment Identifier'),
    Element( u'PLB07-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB07-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB08', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=8,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'S',refdes='',seq=u'09',desc=u'Adjustment Identifier'),
    Element( u'PLB09-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB09-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB10', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=10,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'S',refdes='',seq=u'11',desc=u'Adjustment Identifier'),
    Element( u'PLB11-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB11-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB12', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=12,
    codes=[] ) ),
  Composite( u'C042', Properties(req_sit=u'S',refdes='',seq=u'13',desc=u'Adjustment Identifier'),
    Element( u'PLB13-01', Properties(desc=u'Adjustment Reason Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=0,
      codes=[] ) ),
    Element( u'PLB13-02', Properties(desc=u'Reference Identification', req_sit=u'S', data_type=(u'AN',u'1',u'50'), position=1,
      codes=[] ) ),
  ),
  Element( u'PLB14', Properties(desc=u'Monetary Amount', req_sit=u'S', data_type=(u'R',u'1',u'18'), position=14,
    codes=[] ) ),
),
)
parsed_835_ST_LOOP = Loop( u'ST_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'020',req_sit=u'R',desc=u'Transaction Set Header'),
Segment( u'ST', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Transaction Set Header'),
  Element( u'ST01', Properties(desc=u'Transaction Set Identifier Code', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=1,
    codes=[u'835'] ) ),
  Element( u'ST02', Properties(desc=u'Transaction Set Control Number', req_sit=u'R', data_type=(u'AN',u'4',u'9'), position=2,
    codes=[] ) ),
),
parsed_835_HEADER,
parsed_835_DETAIL,
parsed_835_FOOTER,
Segment( u'SE', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'050',desc=u'Transaction Set Trailer'),
  Element( u'SE01', Properties(desc=u'Number of Included Segments', req_sit=u'R', data_type=(u'N0',u'1',u'10'), position=1,
    codes=[] ) ),
  Element( u'SE02', Properties(desc=u'Transaction Set Control Number', req_sit=u'R', data_type=(u'AN',u'4',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_835_GS_LOOP = Loop( u'GS_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'020',req_sit=u'R',desc=u'Functional Group Header'),
Segment( u'GS', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Functional Group Header'),
  Element( u'GS01', Properties(desc=u'Functional Identifier Code', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'HP'] ) ),
  Element( u'GS02', Properties(desc=u'Application Senders Code', req_sit=u'R', data_type=(u'AN',u'2',u'15'), position=2,
    codes=[] ) ),
  Element( u'GS03', Properties(desc=u'124', req_sit=u'R', data_type=(u'AN',u'2',u'15'), position=3,
    codes=[] ) ),
  Element( u'GS04', Properties(desc=u'Date', req_sit=u'R', data_type=(u'DT',u'8',u'8'), position=4,
    codes=[] ) ),
  Element( u'GS05', Properties(desc=u'Time', req_sit=u'R', data_type=(u'TM',u'4',u'8'), position=5,
    codes=[] ) ),
  Element( u'GS06', Properties(desc=u'Group Control Number', req_sit=u'R', data_type=(u'N0',u'1',u'9'), position=6,
    codes=[] ) ),
  Element( u'GS07', Properties(desc=u'Responsible Agency Code', req_sit=u'R', data_type=(u'ID',u'1',u'2'), position=7,
    codes=[u'X'] ) ),
  Element( u'GS08', Properties(desc=u'Version / Release / Industry Identifier Code', req_sit=u'R', data_type=(u'AN',u'1',u'12'), position=8,
    codes=[u'004010X091A1'] ) ),
),
parsed_835_ST_LOOP,
Segment( u'GE', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'030',desc=u'Functional Group Trailer'),
  Element( u'GE01', Properties(desc=u'97', req_sit=u'R', data_type=(u'N0',u'1',u'6'), position=1,
    codes=[] ) ),
  Element( u'GE02', Properties(desc=u'Group Control Number', req_sit=u'R', data_type=(u'N0',u'1',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_835_ISA_LOOP = Loop( u'ISA_LOOP', Properties(looptype=u'explicit',repeat=u'>1',pos=u'001',req_sit=u'R',desc=u'Interchange Control Header'),
Segment( u'ISA', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'010',desc=u'Interchange Control Header'),
  Element( u'ISA01', Properties(desc=u'I01', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=1,
    codes=[u'00', u'03'] ) ),
  Element( u'ISA02', Properties(desc=u'I02', req_sit=u'R', data_type=(u'AN',u'10',u'10'), position=2,
    codes=[] ) ),
  Element( u'ISA03', Properties(desc=u'I03', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=3,
    codes=[u'00', u'01'] ) ),
  Element( u'ISA04', Properties(desc=u'I04', req_sit=u'R', data_type=(u'AN',u'10',u'10'), position=4,
    codes=[] ) ),
  Element( u'ISA05', Properties(desc=u'I05', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=5,
    codes=[u'01', u'14', u'20', u'27', u'28', u'29', u'30', u'33', u'ZZ'] ) ),
  Element( u'ISA06', Properties(desc=u'I06', req_sit=u'R', data_type=(u'AN',u'15',u'15'), position=6,
    codes=[] ) ),
  Element( u'ISA07', Properties(desc=u'I05', req_sit=u'R', data_type=(u'ID',u'2',u'2'), position=7,
    codes=[u'01', u'14', u'20', u'27', u'28', u'29', u'30', u'33', u'ZZ'] ) ),
  Element( u'ISA08', Properties(desc=u'I07', req_sit=u'R', data_type=(u'AN',u'15',u'15'), position=8,
    codes=[] ) ),
  Element( u'ISA09', Properties(desc=u'I08', req_sit=u'R', data_type=(u'DT',u'6',u'6'), position=9,
    codes=[] ) ),
  Element( u'ISA10', Properties(desc=u'I09', req_sit=u'R', data_type=(u'TM',u'4',u'4'), position=10,
    codes=[] ) ),
  Element( u'ISA11', Properties(desc=u'I10', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=11,
    codes=[u'U'] ) ),
  Element( u'ISA12', Properties(desc=u'I11', req_sit=u'R', data_type=(u'ID',u'5',u'5'), position=12,
    codes=[u'00401'] ) ),
  Element( u'ISA13', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=13,
    codes=[] ) ),
  Element( u'ISA14', Properties(desc=u'I13', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=14,
    codes=[u'0', u'1'] ) ),
  Element( u'ISA15', Properties(desc=u'I14', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=15,
    codes=[u'P', u'T'] ) ),
  Element( u'ISA16', Properties(desc=u'I15', req_sit=u'R', data_type=(u'AN',u'1',u'1'), position=16,
    codes=[] ) ),
),
parsed_835_GS_LOOP,
Segment( u'TA1', Properties(syntax='',req_sit=u'S',repeat=u'1',pos=u'020',desc=u'Interchange Acknowledgement'),
  Element( u'TA101', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=1,
    codes=[] ) ),
  Element( u'TA102', Properties(desc=u'I08', req_sit=u'R', data_type=(u'DT',u'6',u'6'), position=2,
    codes=[] ) ),
  Element( u'TA103', Properties(desc=u'I09', req_sit=u'R', data_type=(u'TM',u'4',u'4'), position=3,
    codes=[] ) ),
  Element( u'TA104', Properties(desc=u'I17', req_sit=u'R', data_type=(u'ID',u'1',u'1'), position=4,
    codes=[u'A', u'E', u'R'] ) ),
  Element( u'TA105', Properties(desc=u'I18', req_sit=u'R', data_type=(u'ID',u'3',u'3'), position=5,
    codes=[u'000', u'001', u'002', u'003', u'004', u'005', u'006', u'007', u'008', u'009', u'010', u'011', u'012', u'013', u'014', u'015', u'016', u'017', u'018', u'019', u'020', u'021', u'022', u'023', u'024', u'025', u'026', u'027', u'028', u'029', u'030', u'031'] ) ),
),
Segment( u'IEA', Properties(syntax='',req_sit=u'R',repeat=u'1',pos=u'030',desc=u'Interchange Control Trailer'),
  Element( u'IEA01', Properties(desc=u'I16', req_sit=u'R', data_type=(u'N0',u'1',u'5'), position=1,
    codes=[] ) ),
  Element( u'IEA02', Properties(desc=u'I12', req_sit=u'R', data_type=(u'N0',u'9',u'9'), position=2,
    codes=[] ) ),
),
)
parsed_835 = Message( u'835', Properties(desc=u'HIPAA Health Care Claim Payment/Advice X091-835'),
parsed_835_ISA_LOOP,
)




#!/usr/bin/python
"""
Generate all parsers from PyX12-1.5.0.zip

Example Usage:
    python generate_all_parsers.py ../../Downloads/pyx12-1.5.0.zip
"""
import argparse
import fnmatch
import logging
import os
import sys
import zipfile

from tigershark.tools import convertPyX12


def convert_from_zip(zip_file_path, dest_path, structure):
    """ Convert all of the xml files in a zip. """
    zipf = zipfile.ZipFile(zip_file_path)

    filenames = [f for f in zipf.namelist() if "map" in f and "xml" in f]
    pyx12_version_str = filenames[0].split("/", 1)[0]
    for filename in filenames:
        if fnmatch.fnmatch(filename.rsplit('/', 1)[1],
                "[0-9][0-9][0-9]*.4010.X*.xml"):
            try:
                data_ele_file = zipf.open("{pyx12}/pyx12/map/dataele.xml".format(
                    pyx12=pyx12_version_str))
                codes_file = zipf.open("{pyx12}/pyx12/map/codes.xml".format(
                    pyx12=pyx12_version_str))
                spec_file = zipf.open(filename)
                dest_fname = os.path.join(dest_path, "M%s.py") % \
                        filename.rsplit('/', 1)[1].rsplit('.', 1)[0].\
                        replace(".", "_")
                parser_name = "parsed_%s" % \
                        filename.rsplit('/', 1)[1].split('.', 1)[0]

                logging.info("Converting file: {filename} to {dest}".format(
                    filename=filename, dest=dest_fname))
                convertPyX12.writeFile(dest_fname, parser_name,
                    convertPyX12.convertFile(spec_file, data_ele_file,
                        codes_file),
                    structure=structure)
                # ZipFile file-like objects don't support seek, so we have to
                # close and re-open them every time :\
                spec_file.close()
                data_ele_file.close()
                codes_file.close()
            except Exception as e:
                raise e
                logging.error(e)

if __name__ == "__main__":
    logging.basicConfig(stream=sys.stdout, level=logging.INFO)
    parser = argparse.ArgumentParser(
        description="Convert all PyX12 XML files to python modules.")
    parser.add_argument('zip_file_path', help="The path to the zip file "\
            "containing the pyX12 X12 spec files to convert")
    parser.add_argument('-d', '--destination_path', default="parsers",
            help="The destination path. Defaults to './parsers/'.")
    parser.add_argument('-s', '--structure', choices=['flat', 'nested'],
            default="flat", help="The structure of the resulting python "\
                    "class. Nested is easier to read, but may not compile "\
                    "due to too many object instantiations in a single "\
                    "call.")
    args = parser.parse_args()
    try:
        os.makedirs(args.destination_path)
    except:
        pass
    convert_from_zip(args.zip_file_path, args.destination_path, args.structure)



#!/usr/bin/env python
"""Defines several modules with classes for X12 message processing.

We have to disentangle two separate concepts.

    -   **Message Type**.  This is the structure of the message.
        The structure is defined by an :class:`X12.parse.Message` object.  In addition
        to defining the structure, a parser can also unmarshall a block of text.
        It can also be used for meta-level processing like emitting DB designs.

    -   **Message Instance**.  This a specific message's data; a collection of
        Segments which can be interpreted as a Message with Loops.  (The Loops
        are not stated, only the Segments.)  A message instance can be built
        by unmarshalling a block of text, or it can be built "manually" using
        object constructors.  A message instance can unmarshall itself, returning
        a block of text.

The :mod:`X12.parse` package defines the classes which are used to build an X12 message
parser (or "unmarshaller").  The classes are generic, and don't recognize any
specific X12 message structure.

A parser for a given message type can be built from the :mod:`X12.parse` classes manually or
by a conversion tool.
The manual construction involves writing a bunch of object constructors to build parsers
for the Message, the Loops within the Message, the Segments within the Loops,
and the Composites and Elements within the Segments.

A conversion tool usually reads some other meta-data version of the message structure,
and builds an :mod:`X12.parse` structure.  There is a conversion tool for :file:`.xml` files.  Both of these conversion tools
build the complete :mod:`X12.parse` object, which can either be used to parse messages,
or can be used for other meta-level processing.

The :mod:`X12.map` package supports meta-level processing.  This includes subclasses
of :class:`X12.parse.StructureVisitor` that can traverse an :mod:`X12.parse` object to
report on its structure.
This is used by the conversion tools to traverse the definitions
and emit the source code, Django class definitions or raw SQL definitions
for the message that the parser recognizes.

The :mod:`X12.message` package defines a message instance.

``map`` sub-package
====================

..  automodule:: X12.map

``message`` sub-package
========================

..  automodule:: X12.message


``file`` module
====================

..  automodule:: X12.file


``parse`` module
====================

..  automodule:: X12.parse
"""
